package futils.filters;

import java.io.File;
import java.io.FileFilter;

/**
 * Copyright DocJava, inc. User: lyon
 * <p/>
 * Date: Jan 8, 2005
 * <p/>
 * Time: 10:46:27 AM
 */
class DateFilter implements FileFilter {
    private final boolean acceptEverything;
    private final boolean acceptAllFiles;
    private final boolean acceptAllDirectories;
    private final boolean acceptFilesAfterDate;
    private final boolean acceptDirsAfterDate;
    private final boolean acceptAllAfterDate;
    private DateFilterBean dateFilterBean;

    public DateFilter(DateFilterBean dateFilterBean, boolean acceptEverything,
                      boolean acceptAllFiles,
                      boolean acceptAllDirectories,
                      boolean acceptFilesAfterDate,
                      boolean acceptDirsAfterDate,
                      boolean acceptAllAfterDate) {
        this.dateFilterBean = dateFilterBean;
        this.acceptEverything = acceptEverything;
        this.acceptAllFiles = acceptAllFiles;
        this.acceptAllDirectories = acceptAllDirectories;
        this.acceptFilesAfterDate = acceptFilesAfterDate;
        this.acceptDirsAfterDate = acceptDirsAfterDate;
        this.acceptAllAfterDate = acceptAllAfterDate;
    }

    public boolean accept(File f) {
        if (acceptEverything)
            return true;
        if (acceptAllFiles)
            return f.isFile();
        if (acceptAllDirectories)
            return f.isDirectory();
        if (acceptFilesAfterDate) {
            if (!f.isFile()) return false;
            return afterDate(f);
        }
        if (acceptDirsAfterDate) {
            if (!f.isDirectory()) return false;
            return afterDate(f);
        }
        if (acceptAllAfterDate) {
            return afterDate(f);
        }
        return false;
    }

    private boolean afterDate(File f) {
        if (f.lastModified() > dateFilterBean.getDate().getTime())
            return true;
        return false;
    }

}
